@@ -1,20 +1,3 @@
-# Copyright (C) 2007-2015 Barry A. Warsaw
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-# USA.
-
 """Portable, NFS-safe file locking with timeouts for POSIX systems.
 
 This code implements an NFS-safe file-based locking algorithm influenced by
@@ -46,17 +29,6 @@ data.  In a distributed (NFS) environment, you also need to make sure that
 your clocks are properly synchronized.
 """
 
-from __future__ import absolute_import, print_function, unicode_literals
-
-__metaclass__ = type
-__all__ = [
-    'LockError',
-    'AlreadyLockedError',
-    'NotLockedError',
-    'Lock',
-    ]
-
-
 import os
 import sys
 import time
@@ -66,18 +38,20 @@ import socket
 import logging
 import datetime
 
-DEFAULT_LOCK_LIFETIME  = datetime.timedelta(seconds=15)
+from logging import NullHandler
+from public import public
+
+
+DEFAULT_LOCK_LIFETIME = datetime.timedelta(seconds=15)
 # Allowable a bit of clock skew.
 CLOCK_SLOP = datetime.timedelta(seconds=10)
-try:
-    MAXINT = sys.maxint
-except AttributeError:
-    # Python 3.
-    MAXINT = sys.maxsize
+MAXINT = sys.maxsize
 
 # Details separator; also used in calculating the claim file path.  Lock files
-# should not include this character.
-SEP = '|'
+# should not include this character.  We do it like this so flake8 won't
+# complain about SEP.
+SEP = ('^' if sys.platform == 'win32' else '|')
+public(SEP=SEP)
 
 # LP: #977999 - catch both ENOENT and ESTALE.  The latter is what an NFS
 # server should return, but some Linux versions return ENOENT.
@@ -88,33 +62,33 @@ log = logging.getLogger('flufl.lock')
 
 # Install a null handler to avoid warnings when applications don't set their
 # own flufl.lock logger.  See http://docs.python.org/library/logging.html
-try:
-    from logging import NullHandler
-except ImportError:
-    # Python < 2.7.
-    class NullHandler(logging.Handler):
-        def emit(self, record):
-            pass
 logging.getLogger('flufl.lock').addHandler(NullHandler())
 
 
-# Exceptions that can be raised by this module
+@public
 class LockError(Exception):
     """Base class for all exceptions in this module."""
 
+
+@public
 class AlreadyLockedError(LockError):
     """An attempt is made to lock an already locked object."""
 
+
+@public
 class NotLockedError(LockError):
     """An attempt is made to unlock an object that isn't locked."""
 
+
+@public
 class TimeOutError(LockError):
     """The timeout interval elapsed before the lock succeeded."""
 
 
+@public
 class Lock:
     """A portable way to lock resources by way of the file system."""
-    def __init__(self, lockfile, lifetime=None):
+    def __init__(self, lockfile, lifetime=None, separator=SEP):
         """Create the resource lock using the given file name and lifetime.
 
         Each process laying claim to this resource lock will create their own
@@ -124,11 +98,18 @@ class Lock:
         :param lockfile: The full path to the lock file.
         :param lifetime: The expected maximum lifetime of the lock, as a
             timedelta.  Defaults to 15 seconds.
+        :param separator: The separator character to use in the lock file's
+            file name.  Defaults to the vertical bar (`|`) on POSIX systems
+            and caret (`^`) on Windows.
         """
+        # This has to be defined before we call _set_claimfile().
+        self._hostname = socket.getfqdn()
         if lifetime is None:
             lifetime = DEFAULT_LOCK_LIFETIME
         self._lockfile = lockfile
         self._lifetime = lifetime
+        # The separator must be set before we claim the lock.
+        self._separator = separator
         self._set_claimfile()
         # For transferring ownership across a fork.
         self._owned = True
@@ -142,6 +123,16 @@ class Lock:
             ('locked' if self.is_locked else 'unlocked'),
             self._lifetime, os.getpid(), id(self))
 
+    @property
+    def hostname(self):
+        """The current machine's host name.
+
+        :return: The current machine's hostname, as used in the `.details`
+            property.
+        :rtype: str
+        """
+        return self._hostname
+
     @property
     def details(self):
         """Details as read from the lock file.
@@ -159,7 +150,8 @@ class Lock:
             raise
         # Rearrange for signature.
         try:
-            lockfile, hostname, pid, random_ignored = filename.split(SEP)
+            lockfile, hostname, pid, random_ignored = filename.split(
+                self._separator)
         except ValueError:
             raise NotLockedError('Details are unavailable')
         return hostname, int(pid), lockfile
@@ -189,7 +181,7 @@ class Lock:
             self._lifetime = lifetime
         # Do we have the lock?  As a side effect, this refreshes the lock!
         if not self.is_locked and not unconditionally:
-            raise NotLockedError('{0}: {1}'.format(repr(self), self._read()))
+            raise NotLockedError('{}: {}'.format(repr(self), self._read()))
 
     def lock(self, timeout=None):
         """Acquire the lock.
@@ -214,18 +206,18 @@ class Lock:
         # very rare occurrence, only happens from cron, and has only(?) been
         # observed on Solaris 2.6.
         self._touch()
-        log.debug('laying claim: {0}'.format(self._lockfile))
-        # For quieting the logging output
+        log.debug('laying claim: {}'.format(self._lockfile))
+        # For quieting the logging output.
         loopcount = -1
         while True:
             loopcount += 1
             # Create the hard link and test for exactly 2 links to the file.
             try:
                 os.link(self._claimfile, self._lockfile)
-                # If we got here, we know we know we got the lock, and never
+                # If we got here, we know we got the lock, and never
                 # had it before, so we're done.  Just touch it again for the
                 # fun of it.
-                log.debug('got the lock: {0}'.format(self._lockfile))
+                log.debug('got the lock: {}'.format(self._lockfile))
                 self._touch()
                 break
             except OSError as error:
@@ -255,9 +247,9 @@ class Lock:
                         self._linkcount))
                 elif self._read() == self._claimfile:
                     # It was us that already had the link.
-                    log.debug('already locked: {0}'.format(self._lockfile))
+                    log.debug('already locked: {}'.format(self._lockfile))
                     raise AlreadyLockedError('We already had the lock')
-                # otherwise, someone else has the lock
+                # Otherwise, someone else has the lock
                 pass
             # We did not acquire the lock, because someone else already has
             # it.  Have we timed out in our quest for the lock?
@@ -270,7 +262,7 @@ class Lock:
             # to avoid race conditions.  (LP: #827052)
             release_time = self._releasetime
             if (release_time != -1 and
-                datetime.datetime.now() > release_time + CLOCK_SLOP):
+                    datetime.datetime.now() > release_time + CLOCK_SLOP):
                 # Yes, so break the lock.
                 self._break()
                 log.error('lifetime has expired, breaking')
@@ -278,7 +270,7 @@ class Lock:
             # and the expected lock lifetime hasn't expired yet either.  So
             # let's wait a while for the owner of the lock to give it up.
             elif not loopcount % 100:
-                log.debug('waiting for claim: {0}'.format(self._lockfile))
+                log.debug('waiting for claim: {}'.format(self._lockfile))
             self._sleep()
 
     def unlock(self, unconditionally=False):
@@ -306,7 +298,7 @@ class Lock:
         except OSError as error:
             if error.errno not in ERRORS:
                 raise
-        log.debug('unlocked: {0}'.format(self._lockfile))
+        log.debug('unlocked: {}'.format(self._lockfile))
 
     @property
     def is_locked(self):
@@ -318,13 +310,11 @@ class Lock:
         # Discourage breaking the lock for a while.
         try:
             self._touch()
-        except OSError as error:
-            if error.errno == errno.EPERM:
-                # We can't touch the file because we're not the owner.  I
-                # don't see how we can own the lock if we're not the owner.
-                return False
-            else:
-                raise
+        except PermissionError:
+            # We can't touch the file because we're not the owner.  I don't see
+            # how we can own the lock if we're not the owner.
+            log.error('No permission to refresh the log')
+            return False
         # XXX Can the link count ever be > 2?
         if self._linkcount != 2:
             return False
@@ -332,15 +322,14 @@ class Lock:
 
     def finalize(self):
         """Unconditionally unlock the file."""
-        log.debug('finalize: {0}'.format(self._lockfile))
+        log.debug('finalize: {}'.format(self._lockfile))
         self.unlock(unconditionally=True)
 
     def __del__(self):
-        log.debug('__del__: {0}'.format(self._lockfile))
+        log.debug('__del__: {}'.format(self._lockfile))
         if self._owned:
             self.finalize()
 
-    # Python 2.5 context manager protocol support.
     def __enter__(self):
         self.lock()
         return self
@@ -386,7 +375,7 @@ class Lock:
             'Unexpected link count: wanted 2, got {0:d}'.format(
                 self._linkcount))
         assert self.is_locked, 'Expected to be locked'
-        log.debug('transferred the lock: {0}'.format(self._lockfile))
+        log.debug('transferred the lock: {}'.format(self._lockfile))
 
     def take_possession(self):
         """Take possession of a lock from another process.
@@ -398,7 +387,7 @@ class Lock:
         # the transfer.
         while self._linkcount != 2 or self._read() != self._claimfile:
             time.sleep(0.25)
-        log.debug('took possession of the lock: {0}'.format(self._lockfile))
+        log.debug('took possession of the lock: {}'.format(self._lockfile))
 
     def disown(self):
         """Disown this lock.
@@ -415,9 +404,9 @@ class Lock:
         # the lock.  We need to watch out for two Lock objects in the same
         # process pointing to the same lock file.  Without this, if you lock
         # lf1 and do not lock lf2, lf2.locked() will still return True.
-        self._claimfile = SEP.join((
+        self._claimfile = self._separator.join((
             self._lockfile,
-            socket.getfqdn(),
+            self.hostname,
             str(pid),
             str(random.randint(0, MAXINT)),
             ))
@@ -425,11 +414,8 @@ class Lock:
     def _write(self):
         """Write our claim file's name to the claim file."""
         # Make sure it's group writable
-        fp = open(self._claimfile, 'w')
-        try:
+        with open(self._claimfile, 'w') as fp:
             fp.write(self._claimfile)
-        finally:
-            fp.close()
 
     @property
     def retry_errnos(self):
